@@ -6,6 +6,8 @@ Information:
 - APP14 documents
   http://partners.adobe.com/public/developer/en/ps/sdk/5116.DCT_Filter.pdf
   http://java.sun.com/j2se/1.5.0/docs/api/javax/imageio/metadata/doc-files/jpeg_metadata.html#color
+- APP12:
+  http://search.cpan.org/~exiftool/Image-ExifTool/lib/Image/ExifTool/TagNames.pod
 
 Author: Victor Stinner
 """
@@ -116,23 +118,49 @@ class JpegChunkApp0(FieldSet):
             yield PaletteRGB(self, "thumb_palette", 256)
             yield RawBytes(self, "thumb_data", thumb_size, "Thumbnail data")
 
+class Ducky(FieldSet):
+    BLOCK_TYPE = {
+        0: "end",
+        1: "Quality",
+        2: "Comment",
+        3: "Copyright",
+    }
+    def createFields(self):
+        yield Enum(UInt16(self, "type"), self.BLOCK_TYPE)
+        if self["type"].value == 0:
+            return
+        yield UInt16(self, "size")
+        size = self["size"].value
+        if size:
+            yield RawBytes(self, "data", size)
+
+class APP12(FieldSet):
+    """
+    The JPEG APP12 "Picture Info" segment was used by some older cameras, and
+    contains ASCII-based meta information.
+    """
+    def createFields(self):
+        yield String(self, "ducky", 5, '"Ducky" string', charset="ASCII")
+        while not self.eof:
+            yield Ducky(self, "item[]")
+
 class StartOfFrame(FieldSet):
     def createFields(self):
         yield UInt8(self, "precision")
-        if self["precision"].value != 8:
-            raise ParserError("JPEG error: Only 8-bit precision is supported")
 
         yield UInt16(self, "height")
         yield UInt16(self, "width")
         yield UInt8(self, "nr_components")
-        if self["nr_components"].value not in (1, 3):
-            raise ParserError("JPEG error: Only gray or true-color JPEGS supported")
 
         for index in range(self["nr_components"].value):
             yield UInt8(self, "component_id[]")
             yield UInt8(self, "high[]")
             yield UInt8(self, "low[]")
 
+class Comment(FieldSet):
+    def createFields(self):
+        yield String(self, "comment", self.size//8, strip="\0")
+
 class AdobeChunk(FieldSet):
     COLORSPACE_TRANSFORMATION = {
         1: "YCbCr (converted from RGB)",
@@ -153,8 +181,6 @@ class AdobeChunk(FieldSet):
 class StartOfScan(FieldSet):
     def createFields(self):
         yield UInt8(self, "nr_components")
-        if self["nr_components"].value not in (1, 3):
-            raise ParserError("JPEG error: Invalid SOS marker")
 
         for index in range(self["nr_components"].value):
             comp_id = UInt8(self, "component_id[]")
@@ -206,9 +232,11 @@ class JpegChunk(FieldSet):
         0xDC: ("nb_line", "Define number of Lines (DNL)", None),
         0xDD: ("restart_interval", "Define Restart Interval (DRI)", RestartInterval),
         0xE0: ("app0", "APP0", JpegChunkApp0),
+        0xE1: ("exif", "Exif metadata", Exif),
+        0xEC: ("app12", "APP12", APP12),
         0xED: ("photoshop", "Photoshop", PhotoshopMetadata),
         0xEE: ("adobe", "Image encoding information for DCT filters (Adobe)", AdobeChunk),
-        0xFE: ("comment[]", "Comment", None),
+        0xFE: ("comment[]", "Comment", Comment),
     }
     START_OF_FRAME = {
         0xC0: u"Baseline",
@@ -231,9 +259,8 @@ class JpegChunk(FieldSet):
     def __init__(self, parent, name, description=None):
         FieldSet.__init__(self, parent, name, description)
         tag = self["type"].value
-        if tag in self.TAG_INFO:
-            self._name, self._description, self._parser = self.TAG_INFO[tag]
-        elif tag == 0xE1:
+        if tag == 0xE1:
+            # Hack for Adobe extension: XAP metadata (as XML)
             bytes = self.stream.readBytes(self.absolute_address + 32, 6)
             if bytes == "Exif\0\0":
                 self._name = "exif"
@@ -241,6 +268,8 @@ class JpegChunk(FieldSet):
                 self._parser = Exif
             else:
                 self._parser = None
+        elif tag in self.TAG_INFO:
+            self._name, self._description, self._parser = self.TAG_INFO[tag]
         else:
             self._parser = None
 
@@ -283,7 +312,13 @@ class JpegFile(Parser):
         if self.stream.readBytes(0, 2) != "\xFF\xD8":
             return "Invalid file signature"
         try:
-            self.readFirstFields(3)
+            for index, field in enumerate(self):
+                chunk_type = field["type"].value
+                if chunk_type not in JpegChunk.TAG_INFO:
+                    return "Unknown chunk type: 0x%02X (chunk #%s)" % (chunk_type, index)
+                if index == 2:
+                    # Only check 3 fields
+                    break
         except HachoirError:
             return "Unable to parse at least three chunks"
         return True
